@@ -20,6 +20,7 @@ use strict;
 use warnings;
 use Carp;
 use LWP;
+use HTML::Entities;
 use URI;
 use HTTP::Request::Common;
 use HTTP::Cookies;
@@ -32,7 +33,7 @@ use Error qw(:try);             # for web services verification (you may comment
 use Data::Dumper;               # dump hashes for debugging
 use File::Temp qw/ tempfile /;  # create temp files
 
-our $VERSION = '1.88';
+our $VERSION = '1.92';
 
 =head1 NAME
 
@@ -905,6 +906,19 @@ sub _http_defaults {
     my $useragent = shift;
     my $case      = shift;
 
+    # Add additional cookies to the cookie jar if specified
+    if($case->{'addcookie'}) {
+        my $cookie_jar = $useragent->cookie_jar();
+        # add cookies to the cookie jar
+        # can add multiple cookies with a pipe delimiter
+        for my $addcookie (split /\|/mx, $case->{'addcookie'}) {
+            my ($ck_version, $ck_key, $ck_val, $ck_path, $ck_domain, $ck_port, $ck_path_spec, $ck_secure, $ck_maxage, $ck_discard) = split(/,/mx, $addcookie);
+            $cookie_jar->set_cookie( $ck_version, $ck_key, $ck_val, $ck_path, $ck_domain, $ck_port, $ck_path_spec, $ck_secure, $ck_maxage, $ck_discard);
+        }
+        $cookie_jar->save();
+        $cookie_jar->add_cookie_header($request);
+    }
+
     # add an additional HTTP Header if specified
     if($case->{'addheader'}) {
         # can add multiple headers with a pipe delimiter
@@ -945,7 +959,10 @@ sub _httppost {
     my $case        = shift;
 
     if($case->{posttype} ) {
-        if($case->{posttype} =~ m~application/x\-www\-form\-urlencoded~mx) {
+        if(   ($case->{posttype} =~ m~application/x\-www\-form\-urlencoded~mx)
+           or ($case->{posttype} =~ m~application/json~mx)
+          )
+        {
             return $self->_httppost_form_urlencoded($useragent, $case);
         }
         elsif($case->{posttype} =~ m~multipart/form\-data~mx) {
@@ -1316,6 +1333,10 @@ sub _parseresponse {
                 $self->{'parsedresult'}->{$type} =
                   $self->_url_escape( $self->{'parsedresult'}->{$type} );
             }
+            if ( $escape eq 'decode' ) {
+                $self->{'parsedresult'}->{$type} =
+                  decode_entities( $self->{'parsedresult'}->{$type} );
+            }
         }
 
         #print "\n\nParsed String: $self->{'parsedresult'}->{$type}\n\n";
@@ -1576,7 +1597,7 @@ sub _url_escape {
     # LWP handles url encoding already, but use this to escape valid chars that LWP won't convert (like +)
     my @return;
     for my $val (@values) {
-        $val =~ s/[^-\w.,!~'()\/\ ]/uc sprintf "%%%02x", ord $&/egmx;
+        $val =~ s/[^-\w.,!~'()\/\ ]/uc sprintf "%%%02x", ord $&/egmx if defined $val;
         push @return, $val;
     }
     return wantarray ? @return : $return[0];
@@ -1968,9 +1989,72 @@ sub _clean_tmp_files {
     return;
 }
 
-=head1 EXAMPLES
+=head1 TEST CASES
 
-=head2 example test case
+=head2 Parameters
+
+=over
+
+=item addcookie
+
+When added to a test case, this adds a cookie to the cookie jar prior to the test case request being sent (i.e. the test case this is attached to will include any cookies specified in this parameter). This is useful for cases where a cookie is set outside of a Set-Cookie directive in the response header. This parameter takes a comma-delimited list of fields that configure the cookie; the fields for this parameter are a direct one-to-one correllation with the parameters to the HTTP::Cookies::set_cookie method. As well, multiple cookies can be defined by separating with a '|' character as with the addheader parameter.
+
+The comma-delimited list of fields are as follows.
+
+addcookie="version,name,value,path,domain,port,path_spec,secure,maxage,discard"
+
+version - Cookie-spec version number
+
+name - Cookie name.
+
+value - Cookie value.
+
+path - The URL path where the cookie is set.
+
+domain - The domain under which the cookie is set.
+
+port - The port on which the cookie is set.
+
+path_spec - Boolean. Set if the cookie is valid only under 'path' or the entire domain.
+
+secure - Boolean. If true (1), the cookie is only sent over secure connections
+
+maxage - The time in seconds the cookie is valid for.
+
+discard - Boolean. Do not send in future requests and destroy upon the next cookie jar save.
+
+=item parseresponse
+
+Parse a string from the HTTP response for use in subsequent requests. This is mostly used for passing Session ID's, but 
+can be applied to any case where you need to pass a dynamically generated value. It takes the arguments in the format
+"leftboundary|rightboundary", and an optional third argument "leftboundary|rightboundary|escape|decode" when you want
+to force escaping of all non-alphanumeric characters (in case there is a wrong configuration of Apache server it will 
+push encoded HTML characters (&#47; = /,  &#58; = :,  ... ) to the Webinject and decode serve to translate them into normal characters. 
+See the "Session Handling and State Management - Parsing Response Data & Embedded Session ID's" section of this manual for details and examples on how to use this parameter.
+
+Note: You may need to prepend a backslash before certain reserved characters when parsing (sorry that is rather vague).
+
+Note: Newlines (\n) are also valid boundaries and are useful when you need to use the end of the line as a boundary.
+
+parseresponse1
+Additional parameter for response parsing.
+
+parseresponse2
+Additional parameter for response parsing.
+
+parseresponse3
+Additional parameter for response parsing.
+
+parseresponse4
+Additional parameter for response parsing.
+
+parseresponse5
+Additional parameter for response parsing. 
+
+=back
+
+
+=head1 EXAMPLE TEST CASE
 
   <testcases>
     <case
